package pps.core.space.service;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.date.DateField;
import cn.hutool.core.date.DateTime;
import cn.hutool.core.date.DateUtil;
import pps.cloud.prediction.service.IPlantPredictedPowerCloudService;
import pps.cloud.prediction.service.IThirdPowerCloudService;
import pps.cloud.prediction.service.data.plant_predicted_power_data.GetPredictedPowerViewInput;
import pps.cloud.prediction.service.data.plant_predicted_power_data.GetPredictedPowerViewOutput;
import pps.cloud.prediction.service.data.third_active_power.GetThirdActivePowerInput;
import pps.cloud.prediction.service.data.third_active_power.GetThirdActivePowerOutput;
import pps.cloud.space.service.ISpaceYesterdayGreenRateCloudService;
import pps.core.common.constant.BusinessConstant;
import pps.core.space.entity.SpaceOptimizeShortDurationView;
import pps.core.space.mapper.SpaceOptimizeShortDurationViewMapper;
import xstartup.annotation.XService;
import xstartup.base.XContext;
import xstartup.data.XListResult;
import xstartup.data.XServiceResult;

import java.util.Collections;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * 昨日绿电占比Cloud模块
 *
 * @author ZWT
 * @date 2024/11/27 15:17
 */
@XService
public class SpaceYesterdayGreenRateCloudServiceImpl extends SpaceOptimizeBaseService implements ISpaceYesterdayGreenRateCloudService {

    /**
     * 昨日绿电占比
     *
     * @param context 上下文
     * @return {@link XServiceResult }
     */
    @Override
    public XServiceResult yesterdayGreenRateJob(XContext context) {
        //取昨日
        DateTime beginOfYesterday = DateUtil.beginOfDay(DateUtil.yesterday());
        //取昨日间开优化井
        SpaceOptimizeShortDurationViewMapper spaceMapper = context.getBean(SpaceOptimizeShortDurationViewMapper.class);
        List<SpaceOptimizeShortDurationView> durationViewList = spaceMapper.selectResultList(SpaceOptimizeShortDurationView.builder()
                .optimizeDate(beginOfYesterday)
                .build());
        Map<String, Map<String, Map<Integer, List<SpaceOptimizeShortDurationView>>>> durationCollect;
        Map<String, List<GetThirdActivePowerOutput>> activePowerMap;
        Map<String, List<GetPredictedPowerViewOutput>> predictionMap;
        if (CollUtil.isEmpty(durationViewList)) {
            durationCollect = Collections.emptyMap();
            activePowerMap = Collections.emptyMap();
            predictionMap = Collections.emptyMap();
        } else {
            //间开区间Map
            durationCollect = durationViewList.stream()
                    .collect(Collectors.groupingBy(SpaceOptimizeShortDurationView::getLineName,
                            Collectors.groupingBy(SpaceOptimizeShortDurationView::getWellNumber,
                                    Collectors.groupingBy(SpaceOptimizeShortDurationView::getIsOptimize))));
            //取昨日实际发电功率
            IThirdPowerCloudService cloudService = context.getBean(IThirdPowerCloudService.class);
            XListResult<GetThirdActivePowerOutput> result = cloudService.queryPhotovoltaicPower30(context, GetThirdActivePowerInput.builder()
                    .stationNames(durationCollect.keySet())
                    .createDate(beginOfYesterday)
                    .build());
            result.throwIfFail();
            List<GetThirdActivePowerOutput> powerList = result.getResult();
            if (CollUtil.isEmpty(powerList)) {
                activePowerMap = Collections.emptyMap();
            } else {
                activePowerMap = powerList.stream()
                        .collect(Collectors.groupingBy(GetThirdActivePowerOutput::getStationName));
            }
            //昨日预测发电功率
            IPlantPredictedPowerCloudService service = context.getBean(IPlantPredictedPowerCloudService.class);
            XListResult<GetPredictedPowerViewOutput> predictionResult = service.queryPredictionPower30(context, GetPredictedPowerViewInput.builder()
                    .stationNames(durationCollect.keySet())
                    .dataDate(beginOfYesterday)
                    .build());
            predictionResult.throwIfFail();
            List<GetPredictedPowerViewOutput> predictionList = predictionResult.getResult();
            if (CollUtil.isEmpty(predictionList)) {
                predictionMap = Collections.emptyMap();
            } else {
                predictionMap = predictionList.stream()
                        .collect(Collectors.groupingBy(GetPredictedPowerViewOutput::getStationName));
            }
        }
        //初始化时间轴
        List<DateTime> rangeToList = DateUtil.rangeToList(DateUtil.offsetDay(BusinessConstant.DATE_FLAG, -1), BusinessConstant.DATE_FLAG, DateField.MINUTE, 30);

        return XServiceResult.OK;
    }
}